@@ -35,11 +35,11 @@
 #include "qsynthAbout.h"
 
 // Timer constant stuff.
-#define QSYNTH_TIMER_MSECS  200
+#define QSYNTH_TIMER_MSECS  100
 #define QSYNTH_DELAY_MSECS  200
 
 // Scale factors.
-#define QSYNTH_GAIN_SCALE   10.0
+#define QSYNTH_GAIN_SCALE   100.0
 #define QSYNTH_REVERB_SCALE 100.0
 #define QSYNTH_CHORUS_SCALE 10.0
 
@@ -59,7 +59,31 @@ static fluid_cmd_handler_t* qsynth_newclient ( void* data, char* )
     return ::new_fluid_cmd_handler((fluid_synth_t*) data);
 }
 
+//-------------------------------------------------------------------------
+// Audio driver processing stub.
 
+int qsynth_process ( void *pvData, int len, int nin, float **in, int nout, float **out )
+{
+    qsynthEngine *pEngine = (qsynthEngine *) pvData;
+	// Call the synthesizer process function to fill
+	// the output buffers with its audio output.
+    if (::fluid_synth_process(pEngine->pSynth, len, nin, in, nout, out) != 0)
+		return -1;
+	// Now find the peak level for this buffer run...
+	if (pEngine == g_pCurrentEngine) {
+		for (int i = 0; i < nout; i++) {
+			float *out_i = out[i];
+			for (int j = 0; j < len; j++) {
+				float fValue = out_i[j];
+				if (pEngine->fMeterValue[i & 1] < fValue)
+					pEngine->fMeterValue[i & 1] = fValue;
+			}
+		}
+	}
+	// Surely a success :)
+	return 0;
+}
+ 
 //-------------------------------------------------------------------------
 // Midi router stubs to have some midi activity feedback.
 
@@ -136,7 +160,7 @@ static int qsynth_handle_midi_event (void *pvData, fluid_midi_event_t *pMidiEven
 //-------------------------------------------------------------------------
 // Scaling & Clipping helpers.
 
-static int qsynth_iscale_clip ( double fScale, double fValue )
+static void qsynth_set_range_value ( QRangeControl *pRange, double fScale, double fValue )
 {
 #ifdef CONFIG_ROUND
     int iValue = (int) ::round(fScale * fValue);
@@ -151,23 +175,25 @@ static int qsynth_iscale_clip ( double fScale, double fValue )
         iValue--;
 #endif
 
-    if (iValue < 0)
-        iValue = 0;
-    else if (iValue > 100)
-        iValue = 100;
+    if (iValue < pRange->minValue())
+        iValue = pRange->minValue();
+    else if (iValue > pRange->maxValue())
+        iValue = pRange->maxValue();
         
-    return iValue;
+    pRange->setValue(iValue);
 }
 
-static double qsynth_fscale_clip ( int iValue, double fScale )
+static double qsynth_get_range_value ( QRangeControl *pRange, double fScale )
 {
-    double fValue = ((double) iValue / fScale);
-
-    const double fMax = (100.0 / fScale);
-    if (fValue < 0.0)
-        fValue = 0.0;
-    else if (fValue > fMax)
-        fValue = fMax;
+    double fValue = ((double) pRange->value() / fScale);
+    
+    double fMinValue = ((double) pRange->minValue() / fScale);
+    double fMaxValue = ((double) pRange->maxValue() / fScale);
+    
+    if (fValue < fMinValue)
+        fValue = fMinValue;
+    else if (fValue > fMaxValue)
+        fValue = fMaxValue;
         
     return fValue;
 }
@@ -213,11 +239,8 @@ void qsynthMainForm::destroy (void)
         qsynthTab *pTab = (qsynthTab *) TabBar->tabAt(i);
         if (pTab) {
             qsynthEngine *pEngine = pTab->engine();
-            if (pEngine) {
-                if (!pEngine->isDefault())
-                    m_pOptions->saveSetup(pEngine->setup(), pEngine->name());
+            if (pEngine)
                 stopEngine(pEngine);
-            }
         }
     }
 
@@ -262,6 +285,7 @@ void qsynthMainForm::setup ( qsynthOptions *pOptions )
     // Set defaults...
     updateMessagesFont();
     updateMessagesLimit();
+    updateOutputMeters();
 
     // Check if we can redirect our own stdout/stderr...
     if (m_pOptions->bStdoutCapture && ::pipe(g_fdStdout) == 0) {
@@ -353,21 +377,23 @@ void qsynthMainForm::playLoadFiles ( qsynthEngine *pEngine, const QStringList& f
         // Is it a soundfont file...
         if (::fluid_is_soundfont(pszFilename)) {
             if (bSetup || pSetup->soundfonts.find(*iter) == pSetup->soundfonts.end()) {
-                appendMessagesColor(sPrefix + tr("Loading soundfont") + ": \"" + *iter + "\"" + sElipsis, "#999933");
+                appendMessagesColor(sPrefix + tr("Loading soundfont: \"%1\"").arg(*iter) + sElipsis, "#999933");
                 if (::fluid_synth_sfload(pEngine->pSynth, pszFilename, 1) >= 0) {
                     iSoundfonts++;
-                    if (!bSetup)
+                    if (!bSetup) {
                         pSetup->soundfonts.append(*iter);
+                        pSetup->bankoffsets.append("0");
+                    }
                 }
-                else appendMessagesError(sPrefix + tr("Failed to load the soundfont") + ": \"" + *iter + "\".");
+                else appendMessagesError(sPrefix + tr("Failed to load the soundfont: \"%1\".").arg(*iter));
             }
         }  // Or is it a bare midifile?
         else if (::fluid_is_midifile(pszFilename) && pEngine->pPlayer) {
-            appendMessagesColor(sPrefix + tr("Playing MIDI file") + ": \"" + *iter + "\"" + sElipsis, "#99cc66");
+            appendMessagesColor(sPrefix + tr("Playing MIDI file: \"%1\"").arg(*iter) + sElipsis, "#99cc66");
             if (::fluid_player_add(pEngine->pPlayer, pszFilename) >= 0)
                 iMidiFiles++;
             else
-                appendMessagesError(sPrefix + tr("Failed to play MIDI file") + ": \"" + *iter + "\".");
+                appendMessagesError(sPrefix + tr("Failed to play MIDI file: \"%1\".").arg(*iter));
         }
     }
     
@@ -486,7 +512,7 @@ void qsynthMainForm::appendMessagesError( const QString& s )
     if (m_pMessagesForm)
         m_pMessagesForm->show();
 
-    appendMessagesColor(s, "#ff0000");
+    appendMessagesColor(s.simplifyWhiteSpace(), "#ff0000");
 
     QMessageBox::critical(this, tr("Error"), s, tr("Cancel"));
 }
@@ -521,14 +547,36 @@ void qsynthMainForm::updateMessagesLimit (void)
 }
 
 
+// Force update of the output meters visibility.
+void qsynthMainForm::updateOutputMeters (void)
+{
+    if (m_pOptions == NULL)
+        return;
+
+    if (m_pOptions->bOutputMeters)
+        OutputGroupBox->show();
+    else
+        OutputGroupBox->hide();
+        
+    adjustSize();
+}
+
+
 // Stabilize current form toggle buttons that may be astray.
 void qsynthMainForm::stabilizeForm (void)
 {
     qsynthEngine *pEngine = currentEngine();
     
-    if (pEngine && pEngine->pSynth) {
-        GainGroupBox->setEnabled(true);
-        ReverbGroupBox->setEnabled(true);
+    bool bEnabled = (pEngine && pEngine->pSynth);
+    
+    GainGroupBox->setEnabled(bEnabled);
+    ReverbGroupBox->setEnabled(bEnabled);
+    ChorusGroupBox->setEnabled(bEnabled);
+    OutputGroupBox->setEnabled(bEnabled && pEngine->bMeterEnabled);
+    ProgramResetPushButton->setEnabled(bEnabled);
+    SystemResetPushButton->setEnabled(bEnabled);
+    
+    if (bEnabled) {
         bool bReverbActive = ReverbActiveCheckBox->isChecked();
         ReverbRoomTextLabel->setEnabled(bReverbActive);
         ReverbDampTextLabel->setEnabled(bReverbActive);
@@ -557,19 +605,11 @@ void qsynthMainForm::stabilizeForm (void)
         ChorusSpeedSpinBox->setEnabled(bChorusActive);
         ChorusDepthSpinBox->setEnabled(bChorusActive);
         ChorusTypeComboBox->setEnabled(bChorusActive);
-        ChorusGroupBox->setEnabled(true);
-        ProgramResetPushButton->setEnabled(true);
-        SystemResetPushButton->setEnabled(true);
-        RestartPushButton->setText(tr("Re&start"));
         qsynthSetup *pSetup = pEngine->setup();
         bool bMidiIn = (pSetup && pSetup->bMidiIn);
         ChannelsPushButton->setEnabled(bMidiIn);
+        RestartPushButton->setText(tr("Re&start"));
     } else {
-        GainGroupBox->setEnabled(false);
-        ReverbGroupBox->setEnabled(false);
-        ChorusGroupBox->setEnabled(false);
-        ProgramResetPushButton->setEnabled(false);
-        SystemResetPushButton->setEnabled(false);
         ChannelsPushButton->setEnabled(false);
         RestartPushButton->setText(tr("&Start"));
     }
@@ -738,10 +778,13 @@ void qsynthMainForm::toggleMessagesForm (void)
 
     if (m_pMessagesForm) {
         m_pOptions->saveWidgetGeometry(m_pMessagesForm);
-        if (m_pMessagesForm->isVisible())
+        if (m_pMessagesForm->isVisible()) {
             m_pMessagesForm->hide();
-        else
+        } else {
             m_pMessagesForm->show();
+            m_pMessagesForm->raise();
+            m_pMessagesForm->setActiveWindow();
+        }
     }
 }
 
@@ -754,10 +797,13 @@ void qsynthMainForm::toggleChannelsForm (void)
 
     if (m_pChannelsForm) {
         m_pOptions->saveWidgetGeometry(m_pChannelsForm);
-        if (m_pChannelsForm->isVisible())
+        if (m_pChannelsForm->isVisible()) {
             m_pChannelsForm->hide();
-        else
+        } else {
             m_pChannelsForm->show();
+            m_pChannelsForm->raise();
+            m_pChannelsForm->setActiveWindow();
+        }
     }
 }
 
@@ -784,6 +830,7 @@ void qsynthMainForm::showOptionsForm (void)
             m_pOptions->sMessagesFont = m_pMessagesForm->messagesFont().toString();
         // To track down deferred or immediate changes.
         QString sOldMessagesFont = m_pOptions->sMessagesFont;
+        bool    bOutputMeters    = m_pOptions->bOutputMeters;
         bool    bStdoutCapture   = m_pOptions->bStdoutCapture;
         bool    bKeepOnTop       = m_pOptions->bKeepOnTop;
         int     bMessagesLimit   = m_pOptions->bMessagesLimit;
@@ -808,6 +855,12 @@ void qsynthMainForm::showOptionsForm (void)
                 (!bMessagesLimit &&  m_pOptions->bMessagesLimit) ||
                 (iMessagesLimitLines !=  m_pOptions->iMessagesLimitLines))
                 updateMessagesLimit();
+            // There's some option(s) that need a global restart...
+            if (( bOutputMeters  && !m_pOptions->bOutputMeters) ||
+                (!bOutputMeters  &&  m_pOptions->bOutputMeters)) {
+                updateOutputMeters();
+                restartAllEngines();
+            }
         }
         // Done.
         delete pOptionsForm;
@@ -832,17 +885,12 @@ void qsynthMainForm::tabSelect ( int iTab )
     if (iTab == m_iCurrentTab)
         return;
 
-    const QString sElipsis = "...";
-    const QString sColon   = ": ";
-
     // Try to save old tab settings...
     qsynthTab *pTab = (qsynthTab *) TabBar->tab(m_iCurrentTab);
     if (pTab) {
         qsynthEngine *pEngine = pTab->engine();
-        if (pEngine) {
-            appendMessages(pEngine->name() + sColon + tr("Saving panel settings") + sElipsis);
+        if (pEngine)
             savePanelSettings(pEngine);
-        }
     }
     
     // Make it official.
@@ -857,7 +905,6 @@ void qsynthMainForm::tabSelect ( int iTab )
             g_pCurrentEngine = pEngine;
             // And do the change.
             setCaption(QSYNTH_TITLE " - " + tr(QSYNTH_SUBTITLE) + " [" + pEngine->name() + "]");
-            appendMessages(pEngine->name() + sColon + tr("Loading panel settings") + sElipsis);
             loadPanelSettings(pEngine, false);
             resetChannelsForm(pEngine, false);
         }
@@ -876,11 +923,11 @@ void qsynthMainForm::tabContextMenu ( qsynthTab *pTab, const QPoint& pos )
         pEngine = pTab->engine();
 
     QPopupMenu* pContextMenu = new QPopupMenu(this);
-    int iNew    = pContextMenu->insertItem(tr("New..."));
-    int iDelete = pContextMenu->insertItem(tr("Delete"));
+    int iNew    = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("add1.png")), tr("New engine..."));
+    int iDelete = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("remove1.png")), tr("Delete"));
     pContextMenu->setItemEnabled(iDelete, pEngine && !pEngine->isDefault());
     pContextMenu->insertSeparator();
-    int iSetup  = pContextMenu->insertItem(tr("Setup..."));
+    int iSetup  = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("setup1.png")), tr("Setup..."));
     pContextMenu->setItemEnabled(iSetup, pEngine);
     int iItemID = pContextMenu->exec(pos);
     delete pContextMenu;
@@ -970,6 +1017,15 @@ void qsynthMainForm::timerSlot (void)
     if (m_iChorusChanged > 0)
         updateChorus();
 
+    // Meter update.
+    if (g_pCurrentEngine->bMeterEnabled) {
+        OutputMeter->setValue(0, g_pCurrentEngine->fMeterValue[0]);
+        OutputMeter->setValue(1, g_pCurrentEngine->fMeterValue[1]);
+        OutputMeter->refresh();
+        g_pCurrentEngine->fMeterValue[0] = 0.0;
+        g_pCurrentEngine->fMeterValue[1] = 0.0;
+    }
+
     // Register for the next timer slot.
     QTimer::singleShot(QSYNTH_TIMER_MSECS, this, SLOT(timerSlot()));
 }
@@ -1005,18 +1061,43 @@ bool qsynthMainForm::startEngine ( qsynthEngine *pEngine )
     appendMessages(sPrefix + tr("Creating synthesizer engine") + sElipsis);
     pEngine->pSynth = ::new_fluid_synth(pSetup->fluid_settings());
     if (pEngine->pSynth == NULL) {
-        appendMessagesError(sPrefix + tr("Failed to create the synthesizer. Cannot continue without it."));
+        appendMessagesError(sPrefix + tr("Failed to create the synthesizer.\n\nCannot continue without it."));
         return false;
     }
 
-    // Load the soundfonts...
-    playLoadFiles(pEngine, pSetup->soundfonts, true);
+    // Load soundfonts...
+    int i = 0;
+    for (QStringList::ConstIterator iter = pSetup->soundfonts.begin(); iter != pSetup->soundfonts.end(); iter++) {
+        char *pszFilename = (char *) (*iter).latin1();
+        // Is it a soundfont file...
+        if (::fluid_is_soundfont(pszFilename)) {
+            int iBankOffset = pSetup->bankoffsets[i].toInt();
+            appendMessagesColor(sPrefix + tr("Loading soundfont: \"%1\" (bank offset %2)").arg(*iter).arg(iBankOffset) + sElipsis, "#999933");
+            int iSFID = ::fluid_synth_sfload(pEngine->pSynth, pszFilename, 1);
+            if (iSFID < 0)
+                appendMessagesError(sPrefix + tr("Failed to load the soundfont: \"%1\".").arg(*iter));
+#ifdef CONFIG_FLUID_BANK_OFFSET
+            else {
+                if (::fluid_synth_set_bank_offset(pEngine->pSynth, iSFID, iBankOffset) < 0)
+                    appendMessagesError(sPrefix + tr("Failed to set bank offset (%1) for soundfont: \"%2\".").arg(iBankOffset).arg(*iter));
+            }
+#endif
+        }
+        i++;
+    }
 
     // Start the synthesis thread...
-    appendMessages(sPrefix + tr("Creating audio driver") + " (" + pSetup->sAudioDriver + ")" + sElipsis);
-    pEngine->pAudioDriver = ::new_fluid_audio_driver(pSetup->fluid_settings(), pEngine->pSynth);
+    appendMessages(sPrefix + tr("Creating audio driver (%1)").arg(pSetup->sAudioDriver) + sElipsis);
+    pEngine->pAudioDriver  = NULL;
+    pEngine->bMeterEnabled = false;
+    if (m_pOptions->bOutputMeters) {
+        pEngine->pAudioDriver  = ::new_fluid_audio_driver2(pSetup->fluid_settings(), qsynth_process, pEngine);
+        pEngine->bMeterEnabled = (pEngine->pAudioDriver != NULL);
+    }
+    if (pEngine->pAudioDriver == NULL)
+        pEngine->pAudioDriver = ::new_fluid_audio_driver(pSetup->fluid_settings(), pEngine->pSynth);
     if (pEngine->pAudioDriver == NULL) {
-        appendMessagesError(sPrefix + tr("Failed to create the audio driver") + " (" + pSetup->sAudioDriver + "). " + tr("Cannot continue without it."));
+        appendMessagesError(sPrefix + tr("Failed to create the audio driver (%1).\n\nCannot continue without it.").arg(pSetup->sAudioDriver));
         stopEngine(pEngine);
         return false;
     }
@@ -1026,20 +1107,20 @@ bool qsynthMainForm::startEngine ( qsynthEngine *pEngine )
         // In dump mode, text output is generated for events going into
         // and out of the router. The example dump functions are put into
         // the chain before and after the router..
-        appendMessages(sPrefix + tr("Creating MIDI router") + " (" + pSetup->sMidiDriver + ")" + sElipsis);
+        appendMessages(sPrefix + tr("Creating MIDI router (%1)").arg(pSetup->sMidiDriver) + sElipsis);
         pEngine->pMidiRouter = ::new_fluid_midi_router(pSetup->fluid_settings(),
             pSetup->bMidiDump ? qsynth_dump_postrouter : qsynth_handle_midi_event,
             (void *) pEngine);
         if (pEngine->pMidiRouter == NULL) {
-            appendMessagesError(sPrefix + tr("Failed to create the MIDI input router") + " (" + pSetup->sMidiDriver + "); " + tr("no MIDI input will be available."));
+            appendMessagesError(sPrefix + tr("Failed to create the MIDI input router (%1).\n\nNo MIDI input will be available.").arg(pSetup->sMidiDriver));
         } else {
             ::fluid_synth_set_midi_router(pEngine->pSynth, pEngine->pMidiRouter);
-            appendMessages(sPrefix + tr("Creating MIDI driver") + " (" + pSetup->sMidiDriver + ")" + sElipsis);
+            appendMessages(sPrefix + tr("Creating MIDI driver (%1)").arg(pSetup->sMidiDriver) + sElipsis);
             pEngine->pMidiDriver = ::new_fluid_midi_driver(pSetup->fluid_settings(),
                 pSetup->bMidiDump ? ::fluid_midi_dump_prerouter : ::fluid_midi_router_handle_midi_event,
                (void *) pEngine->pMidiRouter);
             if (pEngine->pMidiDriver == NULL)
-                appendMessagesError(sPrefix + tr("Failed to create the MIDI driver") + " (" + pSetup->sMidiDriver + "); " + tr("no MIDI input will be available."));
+                appendMessagesError(sPrefix + tr("Failed to create the MIDI driver (%1).\n\nNo MIDI input will be available.").arg(pSetup->sMidiDriver));
         }
     }
 
@@ -1047,7 +1128,7 @@ bool qsynthMainForm::startEngine ( qsynthEngine *pEngine )
     appendMessages(sPrefix + tr("Creating MIDI player") + sElipsis);
     pEngine->pPlayer = ::new_fluid_player(pEngine->pSynth);
     if (pEngine->pPlayer == NULL) {
-        appendMessagesError(sPrefix + tr("Failed to create the MIDI player. Continuing without a player."));
+        appendMessagesError(sPrefix + tr("Failed to create the MIDI player.\n\nContinuing without a player."));
     } else {
         // Play the midi files, if any.
         playLoadFiles(pEngine, pSetup->midifiles, false);
@@ -1059,9 +1140,9 @@ bool qsynthMainForm::startEngine ( qsynthEngine *pEngine )
         appendMessages(sPrefix + tr("Creating server") + sElipsis);
         pEngine->pServer = ::new_fluid_server(pSetup->fluid_settings(), qsynth_newclient, pEngine->pSynth);
         if (pEngine->pServer == NULL)
-            appendMessagesError(sPrefix + tr("Failed to create the server. Continuing without it."));
+            appendMessagesError(sPrefix + tr("Failed to create the server.\n\nContinuing without it."));
 #else
-        appendMessagesError(sPrefix + tr("Server mode disabled. Continuing without it."));
+        appendMessagesError(sPrefix + tr("Server mode disabled.\n\nContinuing without it."));
 #endif
     }
 
@@ -1100,6 +1181,15 @@ void qsynthMainForm::stopEngine ( qsynthEngine *pEngine )
     if (pSetup == NULL)
         return;
 
+    // Only if there's a legal audio driver...
+    if (pEngine->pAudioDriver) {
+        // Before all else save current engine panel settings...
+        if (pEngine == currentEngine())
+            savePanelSettings(pEngine);
+        // Make those settings persist over...
+        m_pOptions->saveSetup(pSetup, pEngine->isDefault() ? QString::null : pEngine->name());
+    }
+
     // Flush anything that maybe pending...
     flushStdoutBuffer();
 
@@ -1145,6 +1235,7 @@ void qsynthMainForm::stopEngine ( qsynthEngine *pEngine )
         appendMessages(sPrefix + tr("Destroying audio driver") + sElipsis);
         ::delete_fluid_audio_driver(pEngine->pAudioDriver);
         pEngine->pAudioDriver = NULL;
+        pEngine->bMeterEnabled = false;
     }
 
     // And finally, destroy the synthesizer engine.
@@ -1158,7 +1249,6 @@ void qsynthMainForm::stopEngine ( qsynthEngine *pEngine )
 
     // Show up our efforts, if we're currently selected :p
     if (pEngine == currentEngine()) {
-        savePanelSettings(pEngine);
         resetChannelsForm(pEngine, true);
         stabilizeForm();
     }
@@ -1171,6 +1261,32 @@ void qsynthMainForm::stopEngine ( qsynthEngine *pEngine )
 }
 
 
+// Restart all synth engines.
+void qsynthMainForm::restartAllEngines (void)
+{
+    // Always prompt user...
+    bool  bRestart = (QMessageBox::warning(this, tr("Warning"),
+            tr("New settings will be effective after\n"
+               "restarting all fluidsynth engines.") + "\n\n" +
+            tr("Please note that this operation may cause\n"
+               "temporary MIDI and Audio disruption.") + "\n\n" +
+            tr("Do you want to restart all engines now?"),
+            tr("Yes"), tr("No")) == 0);
+
+    // Just restart every engine out there...
+    if (bRestart) {
+        // Restarting means stopping an engine...
+        for (int i = 0; i < TabBar->count(); i++) {
+            qsynthTab *pTab = (qsynthTab *) TabBar->tabAt(i);
+            if (pTab)
+                stopEngine(pTab->engine());
+        }
+        // And making room for immediate restart...
+        m_iTimerDelay = 0;
+    }
+}
+
+
 // Start the fluidsynth clone, based on given settings.
 void qsynthMainForm::restartEngine ( qsynthEngine *pEngine )
 {
@@ -1263,8 +1379,8 @@ void qsynthMainForm::loadPanelSettings ( qsynthEngine *pEngine, bool bUpdate )
 {
     if (pEngine == NULL)
         return;
-    if (pEngine->pSynth == NULL)
-        return;
+//  if (pEngine->pSynth == NULL || pEngine->pAudioDriver == NULL)
+//      return;
 
     qsynthSetup *pSetup = pEngine->setup();
     if (pSetup == NULL)
@@ -1280,19 +1396,19 @@ void qsynthMainForm::loadPanelSettings ( qsynthEngine *pEngine, bool bUpdate )
     m_iReverbUpdated = 1;
     m_iChorusUpdated = 1;
 
-    GainDial->setValue(qsynth_iscale_clip(QSYNTH_GAIN_SCALE, pSetup->fGain));
+    qsynth_set_range_value(GainDial, QSYNTH_GAIN_SCALE, pSetup->fGain);
 
     ReverbActiveCheckBox->setChecked(pSetup->bReverbActive);
-    ReverbRoomDial->setValue(qsynth_iscale_clip(QSYNTH_REVERB_SCALE, pSetup->fReverbRoom));
-    ReverbDampDial->setValue(qsynth_iscale_clip(QSYNTH_REVERB_SCALE, pSetup->fReverbDamp));
-    ReverbWidthDial->setValue(qsynth_iscale_clip(QSYNTH_REVERB_SCALE, pSetup->fReverbWidth));
-    ReverbLevelDial->setValue(qsynth_iscale_clip(QSYNTH_REVERB_SCALE, pSetup->fReverbLevel));
+    qsynth_set_range_value(ReverbRoomDial,  QSYNTH_REVERB_SCALE, pSetup->fReverbRoom);
+    qsynth_set_range_value(ReverbDampDial,  QSYNTH_REVERB_SCALE, pSetup->fReverbDamp);
+    qsynth_set_range_value(ReverbWidthDial, QSYNTH_REVERB_SCALE, pSetup->fReverbWidth);
+    qsynth_set_range_value(ReverbLevelDial, QSYNTH_REVERB_SCALE, pSetup->fReverbLevel);
 
     ChorusActiveCheckBox->setChecked(pSetup->bChorusActive);
     ChorusNrDial->setValue(pSetup->iChorusNr);
-    ChorusLevelDial->setValue(qsynth_iscale_clip(QSYNTH_CHORUS_SCALE, pSetup->fChorusLevel));
-    ChorusSpeedDial->setValue(qsynth_iscale_clip(QSYNTH_CHORUS_SCALE, pSetup->fChorusSpeed));
-    ChorusDepthDial->setValue(qsynth_iscale_clip(QSYNTH_CHORUS_SCALE, pSetup->fChorusDepth));
+    qsynth_set_range_value(ChorusLevelDial, QSYNTH_CHORUS_SCALE, pSetup->fChorusLevel);
+    qsynth_set_range_value(ChorusSpeedDial, QSYNTH_CHORUS_SCALE, pSetup->fChorusSpeed);
+    qsynth_set_range_value(ChorusDepthDial, QSYNTH_CHORUS_SCALE, pSetup->fChorusDepth);
     ChorusTypeComboBox->setCurrentItem(pSetup->iChorusType);
 
     // Make them dirty.
@@ -1314,26 +1430,26 @@ void qsynthMainForm::savePanelSettings ( qsynthEngine *pEngine )
 {
     if (pEngine == NULL)
         return;
-    if (pEngine->pSynth == NULL)
+    if (pEngine->pSynth == NULL || pEngine->pAudioDriver == NULL)
         return;
 
     qsynthSetup *pSetup = pEngine->setup();
     if (pSetup == NULL)
         return;
 
-    pSetup->fGain = qsynth_fscale_clip(GainDial->value(), QSYNTH_GAIN_SCALE);
+    pSetup->fGain = qsynth_get_range_value(GainDial, QSYNTH_GAIN_SCALE);
 
     pSetup->bReverbActive = ReverbActiveCheckBox->isChecked();
-    pSetup->fReverbRoom   = qsynth_fscale_clip(ReverbRoomSpinBox->value(),  QSYNTH_REVERB_SCALE);
-    pSetup->fReverbDamp   = qsynth_fscale_clip(ReverbDampSpinBox->value(),  QSYNTH_REVERB_SCALE);
-    pSetup->fReverbWidth  = qsynth_fscale_clip(ReverbWidthSpinBox->value(), QSYNTH_REVERB_SCALE);
-    pSetup->fReverbLevel  = qsynth_fscale_clip(ReverbLevelSpinBox->value(), QSYNTH_REVERB_SCALE);
+    pSetup->fReverbRoom   = qsynth_get_range_value(ReverbRoomDial,  QSYNTH_REVERB_SCALE);
+    pSetup->fReverbDamp   = qsynth_get_range_value(ReverbDampDial,  QSYNTH_REVERB_SCALE);
+    pSetup->fReverbWidth  = qsynth_get_range_value(ReverbWidthDial, QSYNTH_REVERB_SCALE);
+    pSetup->fReverbLevel  = qsynth_get_range_value(ReverbLevelDial, QSYNTH_REVERB_SCALE);
 
     pSetup->bChorusActive = ChorusActiveCheckBox->isChecked();
     pSetup->iChorusNr     = ChorusNrSpinBox->value();
-    pSetup->fChorusLevel  = qsynth_fscale_clip(ChorusLevelSpinBox->value(), QSYNTH_CHORUS_SCALE);
-    pSetup->fChorusSpeed  = qsynth_fscale_clip(ChorusSpeedSpinBox->value(), QSYNTH_CHORUS_SCALE);
-    pSetup->fChorusDepth  = qsynth_fscale_clip(ChorusDepthSpinBox->value(), QSYNTH_CHORUS_SCALE);
+    pSetup->fChorusLevel  = qsynth_get_range_value(ChorusLevelDial, QSYNTH_CHORUS_SCALE);
+    pSetup->fChorusSpeed  = qsynth_get_range_value(ChorusSpeedDial, QSYNTH_CHORUS_SCALE);
+    pSetup->fChorusDepth  = qsynth_get_range_value(ChorusDepthDial, QSYNTH_CHORUS_SCALE);
     pSetup->iChorusType   = ChorusTypeComboBox->currentItem();
 }
 
@@ -1446,7 +1562,7 @@ void qsynthMainForm::updateGain (void)
         return;
     m_iGainUpdated++;
 
-    float fGain = qsynth_fscale_clip(GainSpinBox->value(), QSYNTH_GAIN_SCALE);
+    float fGain = qsynth_get_range_value(GainSpinBox, QSYNTH_GAIN_SCALE);
 
     setEngineGain(pEngine, fGain);
     refreshGain();
@@ -1469,10 +1585,10 @@ void qsynthMainForm::updateReverb (void)
         return;
     m_iReverbUpdated++;
 
-    double fReverbRoom  = qsynth_fscale_clip(ReverbRoomSpinBox->value(), QSYNTH_REVERB_SCALE);
-    double fReverbDamp  = qsynth_fscale_clip(ReverbDampSpinBox->value(), QSYNTH_REVERB_SCALE);
-    double fReverbWidth = qsynth_fscale_clip(ReverbWidthSpinBox->value(), QSYNTH_REVERB_SCALE);
-    double fReverbLevel = qsynth_fscale_clip(ReverbLevelSpinBox->value(), QSYNTH_REVERB_SCALE);
+    double fReverbRoom  = qsynth_get_range_value(ReverbRoomSpinBox,  QSYNTH_REVERB_SCALE);
+    double fReverbDamp  = qsynth_get_range_value(ReverbDampSpinBox,  QSYNTH_REVERB_SCALE);
+    double fReverbWidth = qsynth_get_range_value(ReverbWidthSpinBox, QSYNTH_REVERB_SCALE);
+    double fReverbLevel = qsynth_get_range_value(ReverbLevelSpinBox, QSYNTH_REVERB_SCALE);
 
     setEngineReverb(pEngine, fReverbRoom, fReverbDamp, fReverbWidth, fReverbLevel);
     refreshReverb();
@@ -1496,9 +1612,9 @@ void qsynthMainForm::updateChorus (void)
     m_iChorusUpdated++;
 
     int    iChorusNr    = ChorusNrSpinBox->value();
-    double fChorusLevel = qsynth_fscale_clip(ChorusLevelSpinBox->value(), QSYNTH_CHORUS_SCALE);
-    double fChorusSpeed = qsynth_fscale_clip(ChorusSpeedSpinBox->value(), QSYNTH_CHORUS_SCALE);
-    double fChorusDepth = qsynth_fscale_clip(ChorusDepthSpinBox->value(), QSYNTH_CHORUS_SCALE);
+    double fChorusLevel = qsynth_get_range_value(ChorusLevelSpinBox, QSYNTH_CHORUS_SCALE);
+    double fChorusSpeed = qsynth_get_range_value(ChorusSpeedSpinBox, QSYNTH_CHORUS_SCALE);
+    double fChorusDepth = qsynth_get_range_value(ChorusDepthSpinBox, QSYNTH_CHORUS_SCALE);
     int    iChorusType  = ChorusTypeComboBox->currentItem();
 
     setEngineChorus(pEngine, iChorusNr, fChorusLevel, fChorusSpeed, fChorusDepth, iChorusType);
@@ -1520,7 +1636,7 @@ void qsynthMainForm::refreshGain (void)
 
     float fGain = ::fluid_synth_get_gain(pEngine->pSynth);
 
-    GainDial->setValue(qsynth_iscale_clip(QSYNTH_GAIN_SCALE, fGain));
+    qsynth_set_range_value(GainDial, QSYNTH_GAIN_SCALE, fGain);
 }
 
 
@@ -1538,10 +1654,10 @@ void qsynthMainForm::refreshReverb (void)
     double fReverbWidth = ::fluid_synth_get_reverb_width(pEngine->pSynth);
     double fReverbLevel = ::fluid_synth_get_reverb_level(pEngine->pSynth);
 
-    ReverbRoomDial->setValue(qsynth_iscale_clip(QSYNTH_REVERB_SCALE, fReverbRoom));
-    ReverbDampDial->setValue(qsynth_iscale_clip(QSYNTH_REVERB_SCALE, fReverbDamp));
-    ReverbWidthDial->setValue(qsynth_iscale_clip(QSYNTH_REVERB_SCALE, fReverbWidth));
-    ReverbLevelDial->setValue(qsynth_iscale_clip(QSYNTH_REVERB_SCALE, fReverbLevel));
+    qsynth_set_range_value(ReverbRoomDial,  QSYNTH_REVERB_SCALE, fReverbRoom);
+    qsynth_set_range_value(ReverbDampDial,  QSYNTH_REVERB_SCALE, fReverbDamp);
+    qsynth_set_range_value(ReverbWidthDial, QSYNTH_REVERB_SCALE, fReverbWidth);
+    qsynth_set_range_value(ReverbLevelDial, QSYNTH_REVERB_SCALE, fReverbLevel);
 }
 
 
@@ -1561,9 +1677,9 @@ void qsynthMainForm::refreshChorus (void)
     int    iChorusType  = ::fluid_synth_get_chorus_type(pEngine->pSynth);
 
     ChorusNrDial->setValue(iChorusNr);
-    ChorusLevelDial->setValue(qsynth_iscale_clip(QSYNTH_CHORUS_SCALE, fChorusLevel));
-    ChorusSpeedDial->setValue(qsynth_iscale_clip(QSYNTH_CHORUS_SCALE, fChorusSpeed));
-    ChorusDepthDial->setValue(qsynth_iscale_clip(QSYNTH_CHORUS_SCALE, fChorusDepth));
+    qsynth_set_range_value(ChorusLevelDial, QSYNTH_CHORUS_SCALE, fChorusLevel);
+    qsynth_set_range_value(ChorusSpeedDial, QSYNTH_CHORUS_SCALE, fChorusSpeed);
+    qsynth_set_range_value(ChorusDepthDial, QSYNTH_CHORUS_SCALE, fChorusDepth);
     ChorusTypeComboBox->setCurrentItem(iChorusType);
 }
 
