@@ -294,19 +294,29 @@ void qsynthSetupForm::setup ( qsynthOptions *pOptions, qsynthEngine *pEngine, bo
                     pItem->setPixmap(0, *m_pXpmSoundFont);
                     pItem->setText(0, QString::number(pSoundFont->id));
                     pItem->setText(1, pSoundFont->get_name(pSoundFont));
+#ifdef CONFIG_FLUID_BANK_OFFSET
+                    pItem->setText(2, QString::number(::fluid_synth_get_bank_offset(pEngine->pSynth, pSoundFont->id)));
+                    pItem->setRenameEnabled(2, true);
+#endif
                 }
             }
         }
     } else {
         // Load soundfont view from configuration setup list...
         int i = 0;
-        for (QStringList::Iterator iter = m_pSetup->soundfonts.begin(); iter != m_pSetup->soundfonts.end(); iter++) {
+        for (QStringList::Iterator iter = m_pSetup->soundfonts.begin();
+                iter != m_pSetup->soundfonts.end(); iter++) {
             pItem = new QListViewItem(SoundFontListView, pItem);
             if (pItem) {
                 pItem->setPixmap(0, *m_pXpmSoundFont);
-                pItem->setText(0, QString::number(i++));
+                pItem->setText(0, QString::number(i));
                 pItem->setText(1, *iter);
+#ifdef CONFIG_FLUID_BANK_OFFSET
+                pItem->setText(2, m_pSetup->bankoffsets[i]);
+                pItem->setRenameEnabled(2, true);
+#endif
             }
+            i++;
         }
     }
     SoundFontListView->setUpdatesEnabled(true);
@@ -324,8 +334,11 @@ void qsynthSetupForm::accept (void)
     if (m_iDirtyCount > 0) {
         // Save the soundfont view.
         m_pSetup->soundfonts.clear();
-        for (QListViewItem *pItem = SoundFontListView->firstChild(); pItem; pItem = pItem->nextSibling())
+        m_pSetup->bankoffsets.clear();
+        for (QListViewItem *pItem = SoundFontListView->firstChild(); pItem; pItem = pItem->nextSibling()) {
             m_pSetup->soundfonts.append(pItem->text(1));
+            m_pSetup->bankoffsets.append(pItem->text(2));
+        }
         // Will we have a setup renaming?
         m_pSetup->sDisplayName     = DisplayNameLineEdit->text();
         // Audio settings...
@@ -442,11 +455,13 @@ void qsynthSetupForm::stabilizeForm (void)
     SoundFontOpenPushButton->setEnabled(true);
     QListViewItem *pSelectedItem = SoundFontListView->selectedItem();
     if (pSelectedItem) {
+        SoundFontEditPushButton->setEnabled(pSelectedItem->renameEnabled(2));
         SoundFontRemovePushButton->setEnabled(true);
         SoundFontMoveUpPushButton->setEnabled(pSelectedItem->itemAbove() != NULL);
         SoundFontMoveDownPushButton->setEnabled(pSelectedItem->nextSibling() != NULL);
     } else {
         SoundFontRemovePushButton->setEnabled(false);
+        SoundFontEditPushButton->setEnabled(false);
         SoundFontMoveUpPushButton->setEnabled(false);
         SoundFontMoveDownPushButton->setEnabled(false);
     }
@@ -475,6 +490,8 @@ void qsynthSetupForm::contextMenu( QListViewItem *pItem, const QPoint& pos, int
     iItemID = pContextMenu->insertItem(tr("Open..."), this, SLOT(openSoundFont()));
     pContextMenu->insertSeparator();
     bool bEnabled = (pItem != NULL);
+    iItemID = pContextMenu->insertItem(tr("Edit"), this, SLOT(editSoundFont()));
+    pContextMenu->setItemEnabled(iItemID, bEnabled && pItem->renameEnabled(2));
     iItemID = pContextMenu->insertItem(tr("Remove"), this, SLOT(removeSoundFont()));
     pContextMenu->setItemEnabled(iItemID, bEnabled);
     pContextMenu->insertSeparator();
@@ -540,6 +557,10 @@ void qsynthSetupForm::openSoundFont()
             if (pItem) {
                 pItem->setPixmap(0, *m_pXpmSoundFont);
                 pItem->setText(1, sSoundFont);
+#ifdef CONFIG_FLUID_BANK_OFFSET
+                pItem->setText(2, "0");
+                pItem->setRenameEnabled(2, true);
+#endif
                 pItem->setSelected(true);
                 SoundFontListView->setCurrentItem(pItem);
                 m_pOptions->sSoundFontDir = QFileInfo(sSoundFont).dirPath(true);
@@ -560,6 +581,17 @@ void qsynthSetupForm::openSoundFont()
 }
 
 
+// Edit current selected soundfont.
+void qsynthSetupForm::editSoundFont (void)
+{
+    QListViewItem *pItem = SoundFontListView->selectedItem();
+    if (pItem)
+        pItem->startRename(2);
+    
+    stabilizeForm();
+}
+
+
 // Remove current selected soundfont.
 void qsynthSetupForm::removeSoundFont (void)
 {
@@ -620,5 +652,20 @@ void qsynthSetupForm::moveDownSoundFont (void)
 }
 
 
+// Check soundfont bank offset edit.
+void qsynthSetupForm::itemRenamed ( QListViewItem *pItem, int iCol, const QString& sText )
+{
+    if (iCol == 2) {
+        int iBankOffset = sText.toInt();
+        if (iBankOffset < 0 || iBankOffset > 128)
+            iBankOffset = 0;
+        pItem->setText(iCol, QString::number(iBankOffset));
+        m_iDirtyCount++;
+    }
+
+    stabilizeForm();
+}
+
+
 // end of qsynthSetupForm.ui.h
 
